@@ -74,6 +74,18 @@ where
     }
 }
 
+///Copies a given amount of bytes from `reader` to `writer`.
+pub fn copy_exact<R, W>(reader: &mut R, writer: &mut W, num_bytes: usize) -> io::Result<()>
+where
+    R: Read + ?Sized,
+    W: Write + ?Sized,
+{
+    let mut buf = vec![0u8; num_bytes];
+
+    reader.read_exact(&mut buf)?;
+    writer.write_all(&mut buf)
+}
+
 ///Reads data from `reader` and checks for specified `val`ue. When data contains specified value
 ///or `deadline` is reached, stops reading. Returns read data as array of two vectors: elements
 ///before and after the `val`.
@@ -377,6 +389,7 @@ impl<'a> RequestBuilder<'a> {
     ///let response = RequestBuilder::new(&addr)
     ///    .method(Method::POST)
     ///    .body(body)
+    ///    .header("Content-Length", &body.len())
     ///    .header("Connection", "Close")
     ///    .send(&mut stream, &mut writer)
     ///    .unwrap();
@@ -471,12 +484,35 @@ impl<'a> RequestBuilder<'a> {
         };
         let (res, body_part) = self.read_head(stream, head_deadline)?;
 
-        if self.method != Method::HEAD {
-            writer.write_all(&body_part)?;
+        if self.method == Method::HEAD {
+            return Ok(res);
+        }
 
-            if let Some(timeout) = self.timeout {
-                let deadline = Instant::now() + timeout;
-                copy_with_timeout(stream, writer, deadline)?;
+        if let Some(v) = res.headers().get("Transfer-Encoding") {
+            if *v == "chunked" {
+                let mut dechunked = crate::chunked::Reader::new(body_part.as_slice().chain(stream));
+
+                if let Some(timeout) = self.timeout {
+                    let deadline = Instant::now() + timeout;
+                    copy_with_timeout(&mut dechunked, writer, deadline)?;
+                } else {
+                    io::copy(&mut dechunked, writer)?;
+                }
+
+                return Ok(res);
+            }
+        }
+
+        writer.write_all(&body_part)?;
+
+        if let Some(timeout) = self.timeout {
+            let deadline = Instant::now() + timeout;
+            copy_with_timeout(stream, writer, deadline)?;
+        } else {
+            let num_bytes = res.content_len().unwrap_or(0);
+
+            if num_bytes > 0 {
+                copy_exact(stream, writer, num_bytes - body_part.len())?;
             } else {
                 io::copy(stream, writer)?;
             }
@@ -692,6 +728,7 @@ impl<'a> Request<'a> {
     ///
     ///let response = Request::new(&uri)
     ///    .method(Method::POST)
+    ///    .header("Content-Length", &body.len())
     ///    .body(body)
     ///    .send(&mut writer)
     ///    .unwrap();
@@ -926,6 +963,32 @@ pub fn head<T: AsRef<str>>(uri: T) -> Result<Response, error::Error> {
     Request::new(&uri).method(Method::HEAD).send(&mut writer)
 }
 
+///Creates and sends POST request. Returns response for this request.
+///
+///# Examples
+///```
+///use http_req::request;
+///
+///let mut writer = Vec::new();
+///const uri: &str = "https://www.rust-lang.org/learn";
+///const body: &[u8; 27] = b"field1=value1&field2=value2";
+///
+///let response = request::post(uri, body, &mut writer).unwrap();
+///```
+pub fn post<T: AsRef<str>, U: Write>(
+    uri: T,    
+    body: &[u8],
+    writer: &mut U,
+) -> Result<Response, error::Error> {
+    let uri = uri.as_ref().parse::<Uri>()?;
+
+    Request::new(&uri)
+        .method(Method::POST)
+        .header("Content-Length", &body.len())
+        .body(body)
+        .send(writer)
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -1252,4 +1315,18 @@ mod tests {
         let res = head(URI_S).unwrap();
         assert_ne!(res.status_code(), UNSUCCESS_CODE);
     }
+
+    #[ignore]
+    #[test]
+    fn request_post() {
+        let mut writer = Vec::new();
+        let res = post(URI, &BODY, &mut writer).unwrap();
+
+        assert_ne!(res.status_code(), UNSUCCESS_CODE);
+
+        let mut writer = Vec::with_capacity(200);
+        let res = post(URI_S, &BODY, &mut writer).unwrap();
+
+        assert_ne!(res.status_code(), UNSUCCESS_CODE);
+    }
 }
